import os
import os.path
import json

import gaurabda.GCDisplaySettings as GCDisplaySettings
from gaurabda.GCEnums import MahadvadasiType,EkadasiParanaType,FastType,SpecialFestivalId
import gaurabda.GCUT as GCUT

gstr = {}
gstr_Modified = False

def GetMonthAbreviation(month):
    return getString(64 + month)

def GetTithiName(i):
    return getString(600 + i % 30)

def GetNaksatraName(n):
    return getString(630 + n % 27)

def GetNaksatraChildSylable(n,pada):
    i = (n * 4 + pada) % 108
    childsylable = [
        "chu", "che", "cho", "la", #asvini
        "li", "lu", "le", "lo", # bharani
        "a", "i", "u", "e", #krtika
        "o", "va", "vi", "vo", # rohini
        "ve", "vo", "ka", "ke", # mrgasira
        "ku","gha","ng","chha", # ardra
        "ke","ko","ha","hi", # punarvasu
        "hu","he","ho","da", # pushya
        "di","du","de","do", #aslesa
        "ma","mi","mu","me", #magha
        "mo","ta","ti","tu", #purvaphalguni
        "te","to","pa","pi", #uttaraphalguni
        "pu","sha","na","tha",#hasta
        "pe","po","ra","ri",#chitra
        "ru","re","ra","ta",#svati
        "ti","tu","te","to",#visakha
        "na","ni","nu","ne",# anuradha
        "no","ya","yi","yu",#jyestha
        "ye","yo","ba","bi",# mula
        "bu","dha","bha","dha",#purvasada
        "be","bo","ja","ji",# uttarasada
        "ju","je","jo","gha",#sravana
        "ga","gi","gu","ge",# dhanistha
        "go","sa","si","su",#satabhisda
        "se","so","da","di",#purvabhadra
        "du","tha","jna","da",#uttarabhadra
        "de","do","cha","chi"# revati
    ]

    return childsylable[i]

def GetRasiChildSylable(n):
    childsylable = [
        "a.. e.. la..",
        "u.. ba.. va..",
        "ka.. gha..",
        "da.. ha..",
        "ma..",
        "pa..",
        "ra.. ta..",
        "na.. ya..",
        "dha.. bha... pha..",
        "kha.. ja..",
        "ga.. sa.. sha..",
        "da.. ca.. tha.. jha.."
    ]

    return childsylable[n % 12]

def GetYogaName(n):
    return getString(660 + n % 27)

def GetSankrantiName(i):
    return getString(688 + i % 12)

def GetSankrantiNameEn(i):
    return getString(700 + i % 12)

def GetPaksaChar(i):
    return 'G' if i else 'K'

def GetPaksaName(i):
    if i: return getString(712)
    return getString(713)

def GetMasaName(i):
    masa_format = GCDisplaySettings.getValue(49)
    str_start = 720
    if masa_format==1:
        str_start=897
    elif masa_format==2:
        str_start=871
    elif masa_format==3:
        str_start=884
    return getString(str_start + i%13)

def GetMahadvadasiName(i):
    if i==MahadvadasiType.EV_NULL or i==MahadvadasiType.EV_SUDDHA:
        return None
    elif i==MahadvadasiType.EV_UNMILANI:
        return getString(733)
    elif i==MahadvadasiType.EV_TRISPRSA or i==MahadvadasiType.EV_UNMILANI_TRISPRSA:
        return getString(734)
    elif i==MahadvadasiType.EV_PAKSAVARDHINI:
        return getString(735)
    elif i==MahadvadasiType.EV_JAYA:
        return getString(736)
    elif i==MahadvadasiType.EV_VIJAYA:
        return getString(737)
    elif i==MahadvadasiType.EV_PAPA_NASINI:
        return getString(738)
    elif i==MahadvadasiType.EV_JAYANTI:
        return getString(739)
    elif i==MahadvadasiType.EV_VYANJULI:
        return getString(740)
    else:
        return None

def GetSpecFestivalName(i):
    if i==SpecialFestivalId.SPEC_JANMASTAMI:
        return getString(741)
    elif i==SpecialFestivalId.SPEC_GAURAPURNIMA:
        return getString(742)
    elif i==SpecialFestivalId.SPEC_RETURNRATHA:
        return getString(743)
    elif i==SpecialFestivalId.SPEC_HERAPANCAMI:
        return getString(744)
    elif i==SpecialFestivalId.SPEC_GUNDICAMARJANA:
        return getString(745)
    elif i==SpecialFestivalId.SPEC_GOVARDHANPUJA:
        return getString(746)
    elif i==SpecialFestivalId.SPEC_RAMANAVAMI:
        return getString(747)
    elif i==SpecialFestivalId.SPEC_RATHAYATRA:
        return getString(748)
    elif i==SpecialFestivalId.SPEC_NANDAUTSAVA:
        return getString(749)
    elif i==SpecialFestivalId.SPEC_PRABHAPP:
        return getString(759)
    elif i==SpecialFestivalId.SPEC_MISRAFESTIVAL:
        return getString(750)
    else:
        return getString(64)

def SetSpecFestivalName(i,szName):
    if i==SpecialFestivalId.SPEC_JANMASTAMI:
        setString(741, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_GAURAPURNIMA:
        setString(742, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_RETURNRATHA:
        setString(743, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_HERAPANCAMI:
        setString(744, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_GUNDICAMARJANA:
        setString(745, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_GOVARDHANPUJA:
        setString(746, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_RAMANAVAMI:
        setString(747, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_RATHAYATRA:
        setString(748, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_NANDAUTSAVA:
        setString(749, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_PRABHAPP:
        setString(759, szName)
        gstr_Modified = True
    elif i==SpecialFestivalId.SPEC_MISRAFESTIVAL:
        setString(750, szName)
        gstr_Modified = True

def GetFastingName(i):
    if i == FastType.FAST_NOON:
        return getString(751)
    if i == FastType.FAST_SUNSET:
        return getString(752)
    if i == FastType.FAST_MOONRISE:
        return getString(753)
    if i == FastType.FAST_DUSK:
        return getString(754)
    if i == FastType.FAST_MIDNIGHT:
        return getString(755)
    if i == FastType.FAST_DAY:
        return getString(756)
    return None

def getString(i):
    if isinstance(i,int):
        i = str(i)
    if i in gstr:
        return gstr[i]
    return ''

def GetEkadasiName(nMasa,nPaksa):
    return getString(560 + nMasa*2 + nPaksa)

def GetDSTSignature(nDST):
    if nDST: return 'DST'
    return 'LT'

def GetParanaReasonText(eparana_type):
    if eparana_type == EkadasiParanaType.EP_TYPE_3DAY:
        return getString(165)
    if eparana_type == EkadasiParanaType.EP_TYPE_4TITHI:
        return getString(166)
    if eparana_type == EkadasiParanaType.EP_TYPE_SUNRISE:
        return getString(169)
    if eparana_type == EkadasiParanaType.EP_TYPE_TEND:
        return getString(167)
    if eparana_type == EkadasiParanaType.EP_TYPE_NAKEND:
        return getString(168)
    return ''

def GetVersionText():
    return getString(131)

def setString(i,value):
    if isinstance(i,int):
        i = str(i)
    gstr[i]=value
    gstr_Modified=True

def GetEventClassText(i):
    ect = {
        0: "Appearance Days of the Lord and His Incarnations",
        1: "Events in the Pastimes of the Lord and His Associates",
        2: "Appearance and Disappearance Days of Recent Acaryas",
        3: "Appearance and Disappearance Days of Mahaprabhu's Associates and Other Acaryas",
        4: "ISKCON's Historical Events",
        5: "Bengal-specific Holidays",
        6: "My Personal Events"
    }
    if i in ect: return ect[i]
    return None

def GetDayOfWeek(i):
    return getString(i)

def readFile(pszFile):
    global gstr

    if not os.path.exists(pszFile):
        pszFile = os.path.join(os.path.dirname(__file__), 'res', 'strings.json')
    with open(pszFile,'rt',encoding='utf-8') as json_file:
        gstr = json.load(json_file)

    version = "11, Build 5"
    gstr['130'] = version
    gstr['131'] = 'GCal ' + version
    gstr['132'] = 'Gaurabda Calendar ' + version

def writeFile(pszFile):
    with open(pszFile,'wt',encoding='utf-8') as json_file:
        json_file.write(json.dumps(gstr))

def GetKalaName(i):
    if i == KalaType.KT_RAHU_KALAM:
        return "Rahu kalam"
    if i == KalaType.KT_YAMA_GHANTI:
        return "Yama ghanti"
    if i == KalaType.KT_GULI_KALAM:
        return "Guli kalam"
    if i == KalaType.KT_ABHIJIT:
        return "Abhijit muhurta"
    return ""

def getLongitudeDirectionName(d):
    if d < 0.0:
        return "West"
    return "East"

def getLatitudeDirectionName(d):
    if d < 0.0:
        return "South"
    return "North"

def InitLanguageOutputFromFile(pszFile):
    global gstr
    if os.path.exists(pszFile):
        with open(pszFile,'rt',encoding='utf-8') as json_file:
            g = json.load(json_file)
            for k,v in g.items():
                gstr[k] = v

def getCount():
    return len(gstr)


def unittests():
    GCUT.info('strings')
    readFile('strings.json')
    GCUT.nval(len(gstr),0,'read strings')
    GCUT.val(GetDayOfWeek(0),'Sunday','day of week')
    GCUT.val(GetMasaName(0),'Madhusudana','masa name')



EV_NULL              = 0x100
EV_SUDDHA            = 0x101
EV_UNMILANI          = 0x102
EV_VYANJULI          = 0x103
EV_TRISPRSA          = 0x104
EV_UNMILANI_TRISPRSA = 0x105
EV_PAKSAVARDHINI     = 0x106
EV_JAYA              = 0x107
EV_JAYANTI           = 0x108
EV_PAPA_NASINI       = 0x109
EV_VIJAYA            = 0x110



import math

import gaurabda.GCMath as GCMath
import gaurabda.GCAyanamsha as GCAyanamsha
import gaurabda.GCTimeZone as GCTimeZone
import gaurabda.GCStrings as GCStrings
import gaurabda.GCCoords as GCCoords
import gaurabda.GCUT as GCUT


def calc_epsilon_phi(date):
    arg_mul = [
         [ 0, 0, 0, 0, 1],
         [-2, 0, 0, 2, 2],
         [ 0, 0, 0, 2, 2],
         [ 0, 0, 0, 0, 2],
         [ 0, 1, 0, 0, 0],
         [ 0, 0, 1, 0, 0],
         [-2, 1, 0, 2, 2],
         [ 0, 0, 0, 2, 1],
         [ 0, 0, 1, 2, 2],
         [-2,-1, 0, 2, 2],
         [-2, 0, 1, 0, 0],
         [-2, 0, 0, 2, 1],
         [ 0, 0,-1, 2, 2],
         [ 2, 0, 0, 0, 0],
         [ 0, 0, 1, 0, 1],
         [ 2, 0,-1, 2, 2],
         [ 0, 0,-1, 0, 1],
         [ 0, 0, 1, 2, 1],
         [-2, 0, 2, 0, 0],
         [ 0, 0,-2, 2, 1],
         [ 2, 0, 0, 2, 2],
         [ 0, 0, 2, 2, 2],
         [ 0, 0, 2, 0, 0],
         [-2, 0, 1, 2, 2],
         [ 0, 0, 0, 2, 0],
         [-2, 0, 0, 2, 0],
         [ 0, 0,-1, 2, 1],
         [ 0, 2, 0, 0, 0],
         [ 2, 0,-1, 0, 1],
         [-2, 2, 0, 2, 2],
         [ 0, 1, 0, 0, 1]
        ]
    arg_phi = [
         [-171996,-1742],
         [ -13187,  -16],
         [  -2274,   -2],
         [   2062,    2],
         [   1426,  -34],
         [    712,    1],
         [   -517,   12],
         [   -386,   -4],
         [   -301,    0],
         [    217,   -5],
         [   -158,    0],
         [    129,    1],
         [    123,    0],
         [     63,    0],
         [     63,    1],
         [    -59,    0],
         [    -58,   -1],
         [    -51,    0],
         [     48,    0],
         [     46,    0],
         [    -38,    0],
         [    -31,    0],
         [     29,    0],
         [     29,    0],
         [     26,    0],
         [    -22,    0],
         [     21,    0],
         [     17,   -1],
         [     16,    0],
         [    -16,    1],
         [    -15,    0]
        ]
    arg_eps = [
        [ 92025,   89],
        [  5736,  -31],
        [   977,   -5],
        [  -895,    5],
        [    54,   -1],
        [    -7,    0],
        [   224,   -6],
        [   200,    0],
        [   129,   -1],
        [   -95,    3],
        [     0,    0],
        [   -70,    0],
        [   -53,    0],
        [     0,    0],
        [   -33,    0],
        [    26,    0],
        [    32,    0],
        [    27,    0],
        [     0,    0],
        [   -24,    0],
        [    16,    0],
        [    13,    0],
        [     0,    0],
        [   -12,    0],
        [     0,    0],
        [     0,    0],
        [   -10,    0],
        [     0,    0],
        [    -8,    0],
        [     7,    0],
        [     9,    0]
    ]

    t = ( date -2451545.0)/36525
    delta_phi = 0.0

    # longitude of rising knot
    omega =GCMath.putIn360(125.04452+(-1934.136261+(0.0020708+1.0/450000*t)*t)*t)

    if True:
        l = 280.4665+36000.7698*t
        ls = 218.3165+481267.8813*t

        delta_epsilon = 9.20 * GCMath.cosDeg(omega)+ 0.57* GCMath.cosDeg(2*l)+ 0.10* GCMath.cosDeg(2*ls) - 0.09*GCMath.cosDeg(2*omega)

        delta_phi =(-17.20* GCMath.sinDeg(omega)- 1.32*GCMath.sinDeg(2*l)-0.23*GCMath.sinDeg(2*ls) + 0.21*GCMath.sinDeg(2*omega))/3600
    else:
        # mean elongation of moon to sun
        d = GCMath.putIn360(297.85036+(445267.111480+(-0.0019142+t/189474)*t)*t)

        # mean anomaly of the sun
        m =GCMath.putIn360(357.52772+(35999.050340+(-0.0001603-t/300000)*t)*t)

        # mean anomaly of the moon
        ms =GCMath.putIn360(134.96298+(477198.867398+(0.0086972+t/56250)*t)*t)

        # argument of the latitude of the moon
        f = GCMath.putIn360(93.27191+(483202.017538+(-0.0036825+t/327270)*t)*t)

        delta_phi = 0
        delta_epsilon = 0

        for i in range(31):
            s= arg_mul[i][0]*d + arg_mul[i][1]*m + arg_mul[i][2]*ms + arg_mul[i][3]*f + arg_mul[i][4]*omega
            delta_phi = delta_phi+(arg_phi[i][0]+arg_phi[i][1]*t*0.1)*GCMath.sinDeg(s)
            delta_epsilon = delta_epsilon+(arg_eps[i][0] + arg_eps[i][1]*t*0.1) * GCMath.cosDeg(s)

        delta_phi=delta_phi*0.0001/3600
        delta_epsilon=delta_epsilon*0.0001/3600

    # angle of ecliptic
    epsilon_0=84381.448+(-46.8150+(-0.00059+0.001813*t)*t)*t

    epsilon=(epsilon_0+delta_epsilon)/3600

    return delta_phi, epsilon

def eclipticalToEquatorialCoords(ecc,date):
    eqc = GCCoords.GCEquatorialCoords()
    epsilon = 0.0
    delta_phi = 0.0
    alpha = delta = 0.0

    delta_phi,epsilon = calc_epsilon_phi(date)

    ecc.longitude = GCMath.putIn360(ecc.longitude + delta_phi)

    eqc.rightAscension = GCMath.arcTan2Deg( GCMath.sinDeg(ecc.longitude) * GCMath.cosDeg(epsilon) - GCMath.tanDeg(ecc.latitude) * GCMath.sinDeg(epsilon), GCMath.cosDeg(ecc.longitude));

    eqc.declination = GCMath.arcSinDeg( GCMath.sinDeg(ecc.latitude) * GCMath.cosDeg(epsilon) + GCMath.cosDeg(ecc.latitude) * GCMath.sinDeg(epsilon) * GCMath.sinDeg(ecc.longitude));

    return eqc,ecc


def equatorialToHorizontalCoords(eqc, obs, date):
    hc = GCCoords.GCHorizontalCoords()

    h = GCMath.putIn360(star_time(date) - eqc.rightAscension + obs.longitude_deg)

    hc.azimut = GCMath.rad2deg( math.atan2(GCMath.sinDeg(h), GCMath.cosDeg(h) * GCMath.sinDeg(obs.latitude_deg) - GCMath.tanDeg(eqc.declination) * GCMath.cosDeg(obs.latitude_deg) ))

    hc.elevation = GCMath.rad2deg( math.asin(GCMath.sinDeg(obs.latitude_deg) * GCMath.sinDeg(eqc.declination) + GCMath.cosDeg(obs.latitude_deg) * GCMath.cosDeg(eqc.declination) * GCMath.cosDeg(h)));

    return hc

def GetTextLatitude(d):
    c0 = 'S' if d < 0.0 else 'N'
    d = math.fabs(d)
    a0 = int(math.floor(d))
    a1 = int(math.floor((d - a0)*60 + 0.5))
    return "{}{}{:02d}".format(a0, c0, a1)


def GetTextLongitude(d):
    c0 = 'W' if d < 0.0 else 'E'
    d = math.fabs(d)
    a0 = int(math.floor(d))
    a1 = int(math.floor((d - a0)*60 + 0.5))
    return "{}{}{:02d}".format(a0, c0, a1)


def star_time(date):
    jd = date
    t =(jd-2451545.0)/36525.0
    delta_phi, epsilon = calc_epsilon_phi(date)
    return GCMath.putIn360(280.46061837+360.98564736629*(jd-2451545.0)+
                     t*t*(0.000387933-t/38710000)+
                     delta_phi*GCMath.cosDeg(epsilon) )




class EARTHDATA:
    def __init__(self):
        # observated event
        # 0 - center of the sun
        # 1 - civil twilight
        # 2 - nautical twilight
        # 3 - astronomical twilight
        self.obs = 0
        self.longitude_deg = 0.0
        self.latitude_deg = 0.0
        self.tzone = 0.0
        self.dst = 0

    def __str__(self):
        return '{}: {}  {}: {}  {}: {}'.format(
            GCStrings.getString(10), GetTextLatitude(latitude_deg),
            GCStrings.getString(11), GetTextLongitude(longitude_deg),
            GCStrings.getString(12), GCTimeZone.GetTimeZoneOffsetText(tzone))

    def GetHorizontDegrees(self,jday):
        return GCMath.putIn360(star_time(jday) - self.longitude_deg - GCAyanamsha.GetAyanamsa(jday) + 155)

    def GetNextAscendentStart(self, startDate):
        phi = 30.0
        l1 = l2 = 0.0
        jday = startDate.GetJulianComplete()
        xj = 0.0
        d = GCGregorianDate(date=startDate)
        xd = GCGregorianDate()
        scan_step = 0.05
        prev_tit = 0
        new_tit = -1

        l1 = self.GetHorizontDegrees(jday)

        prev_tit = int(math.floor(l1/phi))

        counter = 0
        while counter < 20:
            xj = jday
            xd.Set(d)

            jday += scan_step
            d.shour += scan_step
            if d.shour > 1.0:
                d.shour -= 1.0
                d.NextDay()

            l2 = self.GetHorizontDegrees(jday)
            new_tit = int(math.floor(l2/phi))

            if prev_tit != new_tit:
                jday = xj
                d.Set(xd)
                scan_step *= 0.5
                counter+=1
                continue
            else:
                l1 = l2

        nextDate = GCGregorianDate.GCGregorianDate(date=d)
        return new_tit, nextDate


def unittests():
    GCUT.info('earth data')
    GCUT.val(GetTextLatitude(12.5),'12N30','text latitude')
    GCUT.val(GetTextLongitude(-15.25),'15W15','text longitude')
    GCUT.val(star_time(2451545.0),280.45704234942144,'start time')
    dp,ep = calc_epsilon_phi(2451545.0)
    GCUT.val(dp,-0.0038975991170544155,'delta phi')
    GCUT.val(ep,23.437690731210242,'epsilon')



import gaurabda.GCStrings as GCStrings
import gaurabda.GCCountry as GCCountry
import gaurabda.GCLocation as GCLocation
import gaurabda.GCLocationList as GCLocationList
import gaurabda.GCDisplaySettings as GCDisplaySettings
import gaurabda.GCEventList as GCEventList
import gaurabda.GCTimeZone as GCTimeZone

GCStrings.readFile('strings.json')
GCCountry.InitWithFile('countries.json')
GCTimeZone.LoadFile('timezones.json')
GCLocationList.OpenFile('locations.json')
GCDisplaySettings.readFile('displays.json')

myLocation = GCLocation.GCLocation(data = {
    'latitude': 27.583,
    'longitude': 77.73,
    'tzid': 188,
    'name': 'Vrndavan, India'
})

lastLocation = GCLocation.GCLocation(data = {
    'latitude': 27.583,
    'longitude': 77.73,
    'tzid': 188,
    'name': 'Vrndavan, India'
})

GCEventList.SetOldStyleFasting(GCDisplaySettings.getValue(42))



import gaurabda.GCStrings as GCStrings
import gaurabda.GCStringBuilder as GCStringBuilder
import gaurabda.GCLayoutData as GCLayoutData
from gaurabda.GCLocation import GCLocation
from gaurabda.GCGregorianDate import GCGregorianDate
from gaurabda.GCDayData import GCDayData, GetFirstDayOfYear
import gaurabda.GCUT as GCUT

class TMasaList:
    def __init__(self):
        self.start = None
        self.end = None
        self.countYears = 0
        self.countMasa = 0
        self.startYear = 0
        self.location = None
        self.arr = []

    def CalculateMasaList(self,loc,year,count):
        day = GCDayData()
        earth = loc.GetEarthData()

        self.startYear = year
        self.countYears = count
        self.start = GCGregorianDate(date = GetFirstDayOfYear(earth, year))
        self.end = GCGregorianDate(date = GetFirstDayOfYear(earth, year + count))
        self.location = GCLocation()
        self.location.Set(loc)

        i = 0
        prev_masa = -1
        prev_paksa = -1
        prev_gyear = -1
        current = 0
        d = GCGregorianDate(date = self.start)

        while d.IsBeforeThis(self.end):
            day.DayCalc(d, earth)
            if prev_paksa != day.nPaksa:
                day.nMasa = day.MasaCalc(d, earth)
                if prev_masa != day.nMasa:
                    if len(self.arr)>0:
                        self.arr[-1]['end'] = GCGregorianDate(date = d, addDays=-1)
                    prev_masa = day.nMasa
                    self.arr.append({
                        'masa': day.nMasa,
                        'masaName': GCStrings.GetMasaName(day.nMasa),
                        'year': day.nGaurabdaYear,
                        'start': GCGregorianDate(date=d)
                    })
            prev_paksa = day.nPaksa
            d.NextDay()

        self.arr[-1]['end'] = GCGregorianDate(date = d, addDays=-1)
        return len(self.arr)

    def formatText(self,stream):
        sb = GCStringBuilder.GCStringBuilder(stream)
        sb.Format = GCStringBuilder.SBTF_TEXT

        stream.write(" {}\r\n\r\n{}: {}\r\n".format(GCStrings.getString(39), GCStrings.getString(40), self.location.m_strFullName))
        stream.write("{} {} {} {}\r\n".format(GCStrings.getString(41), str(self.start), GCStrings.getString(42), str(self.end)))
        stream.write("=" * 65)
        stream.write("\r\n\r\n")

        for m in self.arr:
            stream.write('{:30s}'.format('{} {}'.format(m['masaName'],m['year'])))
            stream.write('   {} - {}\r\n'.format(str(m['start']).rjust(12,' '), str(m['end']).rjust(12,' ')))

        sb.AppendNote()
        sb.AppendDocumentTail()
        return 1

    def formatRtf(self,stream):
        sb = GCStringBuilder.GCStringBuilder(stream)
        sb.Format = GCStringBuilder.SBTF_RTF
        sb.fontSizeH1 = GCLayoutData.textSizeH1
        sb.fontSizeH2 = GCLayoutData.textSizeH2
        sb.fontSizeText = GCLayoutData.textSizeText
        sb.fontSizeNote = GCLayoutData.textSizeNote

        sb.AppendDocumentHeader()

        stream.write("{{\\fs{}\\f2 {} }}\\par\\tx{}\\tx{}\\f2\\fs{}\r\n\\par\r\n{}: {}\\par\r\n".format(GCLayoutData.textSizeH1 , GCStrings.getString(39), 1000*GCLayoutData.textSizeText/24, 4000*GCLayoutData.textSizeText/24 , GCLayoutData.textSizeText, GCStrings.getString(40), self.location.m_strFullName))
        stream.write("{} {} {} {}\\par\r\n".format( GCStrings.getString(41), str(self.start) , GCStrings.getString(42), str(self.end)))
        sb.AppendSeparatorWithWidth(65)
        sb.AppendLine()
        sb.AppendLine()

        for m in self.arr:
            stream.write('\\tab {} {}\\tab '.format(m['masaName'],m['year']))
            stream.write('{} - '.format(str(m['start'])))
            stream.write('{}\\par\r\n'.format(str(m['end'])))

        sb.AppendNote()
        sb.AppendDocumentTail()

        return 1


    def writeHtml(self,stream):
        stream.write("<html>\n<head>\n<title>Masa List</title>\n\n")
        stream.write("<style>\n<!--\nbody {\n  font-family:Verdana;\n  font-size:11pt;\n}\n\ntd.hed {\n  font-size:11pt;\n  font-weight:bold;\n")
        stream.write("  background:#aaaaaa;\n  color:white;\n  text-align:center;\n  vertical-align:center;\n  padding-left:15pt;\n  padding-right:15pt;\n")
        stream.write("  padding-top:5pt;\n  padding-bottom:5pt;\n}\n-->\n</style>\n")
        stream.write("</head>\n")
        stream.write("<body>\n\n")

        stream.write("<p style=\'text-align:center\'><span style=\'font-size:14pt\'>Masa List</span></br>{}: {}</p>\n".format(GCStrings.getString(40), self.location.m_strFullName))
        stream.write("<p align=center>{} {} {} {} </p>\n".format(GCStrings.getString(41), str(self.start), GCStrings.getString(42), str(self.end)))
        stream.write("<hr width=\"50%\">")

        stream.write("<table align=center>")
        stream.write("<tr><td class=\"hed\" style=\'text-align:left\'>MASA NAME&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td><td class=\"hed\">START</td><td class=\"hed\">END</td></tr>")

        for m in self.arr:
            stream.write("<tr>")
            stream.write("<td>{} {}&nbsp;&nbsp;&nbsp;&nbsp;</td>".format( m['masaName'], m['year']))
            stream.write("<td>{}</td>".format(str(m['start'])))
            stream.write("<td>{}</td>".format(str(m['end'])))
            stream.write("</tr>")

        stream.write("</table>")
        stream.write("<hr width=\"50%%\">\n<p align=center>Generated by {}</p>".format(GCStrings.getString(130)))
        stream.write("</body></html>")
        return 1


def unittests():
    tm = TMasaList()
    loc = GCLocation(data={
        'latitude': 48.150002,
        'longitude': 17.116667,
        'tzid': 321,
        'name': 'Bratislava, Slovakia'
    })
    tm.CalculateMasaList(loc,2020,2)

    with open('test/masalist.rtf','wt') as wf:
        tm.formatRtf(wf)
    with open('test/masalist.html','wt') as wf:
        tm.writeHtml(wf)
